import java.util.Date;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.MappedSuperclass;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
@Table(name = "EMP")
@Inheritance(strategy=InheritanceType.TABLE_PER_CLASS)
@DiscriminatorColumn(name = "EMP_TYPE")
public abstract class Employee {
  @Id
  private int id;

  private String name;

  @Temporal(TemporalType.DATE)
  @Column(name = "S_DATE")
  private Date startDate;

  public int getId() {
    return id;
  }

  public void setId(int id) {
    this.id = id;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public Date getStartDate() {
    return startDate;
  }

  public void setStartDate(Date startDate) {
    this.startDate = startDate;
  }

  public String toString() {
    return "Employee id: " + getId() + " name: " + getName();
  }
}

@MappedSuperclass
abstract class AbstractEmployee extends Employee {
  private int vacation;

  public int getVacation() {
    return vacation;
  }

  public void setVacation(int vacation) {
    this.vacation = vacation;
  }
}

@Entity
@DiscriminatorValue("FTEmp")
class FullTimeEmployee extends AbstractEmployee {
  private long salary;

  private long pension;

  public long getPension() {
    return pension;
  }

  public void setPension(long pension) {
    this.pension = pension;
  }

  public long getSalary() {
    return salary;
  }

  public void setSalary(long salary) {
    this.salary = salary;
  }

  public String toString() {
    return "FullTimeEmployee id: " + getId() + " name: " + getName();
  }
}

@Entity(name = "PTEmp")
@DiscriminatorValue("PTEmp")
class PartTimeEmployee extends AbstractEmployee {
  @Column(name = "H_RATE")
  private float hourlyRate;

  public float getHourlyRate() {
    return hourlyRate;
  }

  public void setHourlyRate(float hourlyRate) {
    this.hourlyRate = hourlyRate;
  }

  public String toString() {
    return "PartTimeEmployee id: " + getId() + " name: " + getName();
  }
}
